@@ -16,6 +16,8 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
+#[cfg(not(has_std))]
+use std::vec::Vec;
 
 // an improvement in terms of complexity would be to use a bare HashMap
 // as vec instead of the IndexMap
@@ -23,13 +25,13 @@ use crate::iterators::*;
 
 use std::borrow::Borrow;
 use std::cmp::{Eq, Ord};
+#[cfg(has_std)]
 use std::collections::hash_map::RandomState;
 use std::hash::{BuildHasher, Hash};
-use std::iter::Iterator;
+use std::iter::{Iterator, FromIterator, IntoIterator, Extend};
 use std::mem::{replace, swap};
 
 use indexmap::map::{IndexMap, MutableKeys};
-use take_mut::take;
 
 /// A priority queue with efficient change function to change the priority of an
 /// element.
@@ -41,6 +43,7 @@ use take_mut::take;
 /// Implemented as a heap of indexes, stores the items inside an `IndexMap`
 /// to be able to retrieve them quickly.
 #[derive(Clone)]
+#[cfg(has_std)]
 pub struct PriorityQueue<I, P, H = RandomState>
 where
     I: Hash + Eq,
@@ -53,6 +56,20 @@ where
     size: usize, // The size of the heap
 }
 
+#[derive(Clone)]
+#[cfg(not(has_std))]
+pub struct PriorityQueue<I, P, H>
+where
+    I: Hash + Eq,
+    P: Ord,
+{
+    pub(crate) map: IndexMap<I, P, H>, // Stores the items and assign them an index
+    heap: Vec<usize>,                  // Implements the heap of indexes
+    qp: Vec<usize>,                    // Performs the translation from the index
+    // of the map to the index of the heap
+    size: usize, // The size of the heap
+}
+
 // do not [derive(Eq)] to loosen up trait requirements for other types and impls
 impl<I, P, H> Eq for PriorityQueue<I, P, H>
 where
@@ -73,6 +90,7 @@ where
     }
 }
 
+#[cfg(has_std)]
 impl<I, P> PriorityQueue<I, P>
 where
     P: Ord,
@@ -199,6 +217,7 @@ where
         self.map.capacity()
     }
 }
+
 impl<I, P, H> PriorityQueue<I, P, H>
 where
     P: Ord,
@@ -347,7 +366,7 @@ where
     ///
     /// Computes in **O(log(N))** time.
     pub fn push_increase(&mut self, item: I, priority: P) -> Option<P> {
-        if self.get(&item).map_or(true, |(_, p)| priority > *p) {
+        if self.get_priority(&item).map_or(true, |p| priority > *p) {
             self.push(item, priority)
         } else {
             Some(priority)
@@ -369,7 +388,7 @@ where
     ///
     /// Computes in **O(log(N))** time.
     pub fn push_decrease(&mut self, item: I, priority: P) -> Option<P> {
-        if self.get(&item).map_or(true, |(_, p)| priority < *p) {
+        if self.get_priority(&item).map_or(true, |p| priority < *p) {
             self.push(item, priority)
         } else {
             Some(priority)
@@ -427,12 +446,12 @@ where
     where
         I: Borrow<Q>,
         Q: Eq + Hash,
-        F: FnOnce(P) -> P,
+        F: FnOnce(&mut P),
     {
         let mut pos = 0;
         let mut found = false;
-        if let Some((index, _, p)) = self.map.get_full_mut(item) {
-            take(p, priority_setter);
+        if let Some((index, _, mut p)) = self.map.get_full_mut(item) {
+            priority_setter(&mut p);
             pos = unsafe { *self.qp.get_unchecked(index) };
             found = true;
         }
@@ -494,6 +513,48 @@ where
         self.map.get_full_mut2(item).map(|(_, k, v)| (k, &*v))
     }
 
+    /// Remove an arbitrary element from the priority queue.
+    /// Returns the (item, priority) couple or None if the item
+    /// is not found in the queue.
+    ///
+    /// The operation is performed in **O(log(N))** time (worst case).
+    pub fn remove<Q>(&mut self, item: &Q) -> Option<(I, P)>
+    where
+        I: Borrow<Q>,
+        Q: Eq + Hash,
+    {
+        let element = self.map.swap_remove_full(item);
+        if let Some((i, _, _)) = element {
+            self.size -= 1;
+
+            let pos = self.qp.swap_remove(i);
+            self.heap.swap_remove(pos);
+            if i < self.size {
+                unsafe{
+                    let qpi = self.qp.get_unchecked_mut(i);
+                    if *qpi == self.size {
+                        *qpi = pos;
+                    } else {
+                        *self.heap.get_unchecked_mut(*qpi) = i;
+                    }
+                }
+            }
+            if pos < self.size {
+                unsafe {
+                    let heap_pos = self.heap.get_unchecked_mut(pos);
+                    if *heap_pos == self.size {
+                        *heap_pos = i;
+                    } else {
+                        *self.qp.get_unchecked_mut(*heap_pos) = pos;
+                    }
+                }
+                self.heapify(pos);
+            }
+        }
+
+        element.map(|(_, i, p)| (i, p))
+    }
+
     /// Returns the items not ordered
     pub fn into_vec(self) -> Vec<I> {
         self.map.into_iter().map(|(i, _)| i).collect()
@@ -548,7 +609,7 @@ where
     /// if other is longer than self
     pub fn append(&mut self, other: &mut Self) {
         if other.size > self.size {
-            ::std::mem::swap(self, other);
+            std::mem::swap(self, other);
         }
         if other.size == 0 {
             return;
@@ -588,6 +649,7 @@ where
     /// Remove and return the element with the max priority
     /// and swap it with the last element keeping a consistent
     /// state.
+    ///
     /// Computes in **O(1)** time (average)
     fn swap_remove(&mut self) -> Option<(I, P)> {
         // swap_remove the head
@@ -690,13 +752,14 @@ where
 
 //FIXME: fails when the vector contains repeated items
 // FIXED: repeated items ignored
-impl<I, P> From<Vec<(I, P)>> for PriorityQueue<I, P>
+impl<I, P, H> From<Vec<(I, P)>> for PriorityQueue<I, P, H>
 where
     I: Hash + Eq,
     P: Ord,
+    H: BuildHasher + Default,
 {
     fn from(vec: Vec<(I, P)>) -> Self {
-        let mut pq = Self::with_capacity(vec.len());
+        let mut pq = Self::with_capacity_and_hasher(vec.len(), <_>::default());
         let mut i = 0;
         for (item, priority) in vec {
             if !pq.map.contains_key(&item) {
@@ -715,10 +778,11 @@ where
 //FIXME: fails when the iterator contains repeated items
 // FIXED: the item inside the pq is updated
 // so there are two functions with different behaviours.
-impl<I, P> ::std::iter::FromIterator<(I, P)> for PriorityQueue<I, P>
+impl<I, P, H> FromIterator<(I, P)> for PriorityQueue<I, P, H>
 where
     I: Hash + Eq,
     P: Ord,
+    H: BuildHasher + Default,
 {
     fn from_iter<IT>(iter: IT) -> Self
     where
@@ -727,11 +791,11 @@ where
         let iter = iter.into_iter();
         let (min, max) = iter.size_hint();
         let mut pq = if let Some(max) = max {
-            Self::with_capacity_and_default_hasher(max)
+            Self::with_capacity_and_hasher(max, <_>::default())
         } else if min > 0 {
-            Self::with_capacity_and_default_hasher(min)
+            Self::with_capacity_and_hasher(min, <_>::default())
         } else {
-            Self::with_default_hasher()
+            Self::with_hasher(<_>::default())
         };
         for (item, priority) in iter {
             if pq.map.contains_key(&item) {
@@ -750,7 +814,6 @@ where
     }
 }
 
-use ::std::iter::IntoIterator;
 impl<I, P, H> IntoIterator for PriorityQueue<I, P, H>
 where
     I: Hash + Eq,
@@ -793,7 +856,7 @@ where
     }
 }
 
-impl<I, P, H> ::std::iter::Extend<(I, P)> for PriorityQueue<I, P, H>
+impl<I, P, H> Extend<(I, P)> for PriorityQueue<I, P, H>
 where
     I: Hash + Eq,
     P: Ord,
